<?php
require_once("utilities.php");
require_once("category.php");
require_once('facebook_app.php');
FacebookApp::instance()->ensureFacebookUserLoggedInToVB();

final class CategoryStream {

    // + Singleton implementation
    private static $instance = null;
    public static function instance() {
        if(!isset(self::$instance)) {
            self::$instance = new CategoryStream();
        }
        return self::$instance;
    }
	private function __construct() {
        
    }
    // - Singleton implementation
    
    private $data;
    private $expires_timestamp = 0;
    
    public function renderStream($category) {
        list($selected_mod, $selected_type) = explode('_', $category);
        $data = $this->buildData($category);
        $out = '';
        foreach($data as $data_string => $data_weight) {
            $out = $out . $data_string;
        }
		
		//
		//	When we have a filter that includes multiple content
		//	types, we want to make sure that the user can
		//	differentiate between them, so we need to add
		//	some design element.
		//
		//	We do some dom manipulation on the client to
		//	avoid having to replicate the view code
		//	just to add a styled string to the front of titles.
		//
		//	In the future, we can actually split the view
		//	code, once more drastic changes are needed.
		//
		if($selected_type == 'All') {
			$contenttypes = array(); // Hash to only touch each type once
			echo '	<script>
						$(function() {
						$("li.category_data_item").each(function() {
			';
			foreach(Category::all() as $category1) {
				$contenttype = $category1->getContenttype();
				$display = rtrim($contenttype, 's') . ':';
				if($contenttype != 'All' && !@$contenttypes[$contenttype]) {
					if($contenttype != 'Photos') {
						echo '
							if($(this).attr("data-category").split("_")[1] == "'. $contenttype .'") {
								$(this).children(".title").prepend("<span class=\"contenttype\">'.$display.'</span>");
							}';
					}
					$contenttypes[$contenttype] = true;
				}
			}
			echo '})});</script>
			';							
		}
        echo Category::registry($category)->getDashboardContainerHtml($out);

        return $this->expires_timestamp;
    }
    
    public function buildData($category) {
        $chunk_array = explode('_', $category);
        $data = array();
        $used_categories = array();
		
        //
        // 	Check if this is a variadic category, or category has its own implementation, if so, do not do aggregation.
		//
		$category_impl = Category::registry($category);
        if(count($chunk_array) > 2 || (isset($category_impl) && (count($category_impl->getData()) > 0))) {
			$used_categories[] = $category_impl;
        } else {
            $selected_mod = $chunk_array[0];
            $selected_type = $chunk_array[1];
            
            $categories = Category::all();
			$criteria = ($selected_mod == 'All' ? '' : $selected_mod) . '_' . ($selected_type == 'All' ? '' : $selected_type);                        

			foreach($categories as $category) {
				if($category->showAsFilter() && strstr($category->getCategoryName(), $criteria)) {
					$used_categories[] = $category;               
				}
			}
        }
        foreach($used_categories as $category) {
            foreach($category->getData() as $index => $data_item) {
                $final_weight = $category->getWeight() * $data_item->getRelevance();
                $item_html = $category->getStreamContainerHtml($data_item->getStreamHtml());
                $data[$item_html] = array($final_weight, $category->getContentType(), $data_item->getId());
            }
        }
        $this->expires_timestamp = Category::expires($used_categories);
        
		uasort($data, array("CategoryStream", "list_item_sorter"));
		
		$hash = array();
		foreach($data as $list_item_html => $list_item_meta) {
			if(!isset($hash[$list_item_meta[1]][$list_item_meta[2]])) {
				$hash[$list_item_meta[1]][$list_item_meta[2]] = true;
			} else {
				unset($data[$list_item_html]);
			}
		}
    
        return $data;
    }
    
	private function list_item_sorter($list_item1, $list_item2) {
        $weight1 = $list_item1[0];
        $weight2 = $list_item2[0];
        if($weight1 > $weight2) return -1;
        else if($weight1 == $weight2) return 0;
        else return 1;
    }
	
    public function renderFilters($category) {
        
        list($selected_mod, $selected_type) = explode('_', $category);
        
        $mod_arr = array();
        $type_arr = array();

        foreach(Category::all() as $category) {
            if($category->showAsFilter()) {
                list($mod, $type) = explode("_", $category->getCategoryName());
                $mod_arr[$mod][$type] = $type;
                $type_arr[$type][$mod] = $mod;
            }
        }
        echo '<span>Sort by:</span>';
        echo '<select id="filtermodifier">';               
        foreach(array_keys($mod_arr) as $mod) {
            $selected = (($mod == $selected_mod) ? "selected=\"true\"" : "");
            echo '<option '.$selected.'>'.$mod.'</option>';
        }
        echo '</select>';
		echo '<span>View:</span>';
        echo '<select id="filtertype">';       
        foreach(array_keys($type_arr) as $type) {
            $selected = (($type == $selected_type) ? "selected=\"true\"" : "");
            echo '<option '.$selected.'>'.$type.'</option>';
        }
        echo '</select>';
		
		echo '<script>					
				$(function() {
					$(\'#filtermodifier\').live("change", function(e) {
						var old_type =  $(\'#filtertype\').val();
						$(\'#filtertype\').find(\'option\').remove();
						var mod_arr = ' . json_encode($mod_arr) . ';                      
						var new_mods = mod_arr[$(this).val()];                    
						for(var mod in new_mods) {
							var selected = (old_type == new_mods[mod]) ? "selected=\"true\"" : "";
							$(\'#filtertype\').append("<option " + selected + ">"+new_mods[mod]+"</option>");
						}
						var mod = $("#filtermodifier").val();
						var type = $("#filtertype").val();
						top.location.href = "'. FACEBOOK_CANVAS_URL .'category_stream.php?category=" + mod + "_" + type;
					});
					$(\'#filtertype\').live("change", function(e) {
						var old_mod =  $(\'#filtermodifier\').val();
						$(\'#filtermodifier\').find(\'option\').remove();
						var type_arr = ' . json_encode($type_arr) . ';                      
						var new_types = type_arr[$(this).val()];
						for(var type in new_types) {
							var selected = (old_mod == new_types[type]) ? "selected=\"true\"" : "";
							$(\'#filtermodifier\').append("<option " + selected + ">"+new_types[type]+"</option>");
						}
						var mod = $("#filtermodifier").val();
						var type = $("#filtertype").val();
						top.location.href = "'. FACEBOOK_CANVAS_URL .'category_stream.php?category=" + mod + "_" + type;
					});
				}); // Wait for script to load.
			</script>';
    }
}
if(!isset($_GET['category'])) {
    header('HTTP/1.1 404 Not Found');
    header('Connection: close');
    exit;
}

$category = $_GET['category'];

if(!isset($_GET['do'])) {
    display_header(array('_css/editor.css'), array('_js/jquery.waitforimages.js', '_js/editor.js', '_js/stream.js'));
?>
<div id="dbcontent">
<?php
if(Category::registry($category)->showAsFilter()) {
?>
    <div class="filterheader"><div class="wrapper"><div class="wrapper2">
        <?php           
            CategoryStream::instance()->renderFilters($category);
        ?>
        <div class="loading"></div>
    </div></div></div>
<?php
}
?>

    <div id="box_view" class="stream">
        <?php $expires_timestamp = CategoryStream::instance()->renderStream($category); ?>
        <div class="clear"></div>
    </div>
</div>
<?php 
    display_footer($expires_timestamp);
} else if ($_GET['do'] == 'update') {
    cache_start();
    cache_end(CategoryStream::instance()->renderStream($category));
}

?>